#!/usr/bin/env python

# Rnmap console client v. 0.1.7
#
# Copyright (C) 2000,2001 Tuomo Makinen (tmakinen@pp.htv.fi)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Rnmap cli client v. 0.1.7

Copyright (C) 2000,2001 Tuomo Makinen
Redistributable under the terms of the GNU General Public License

Usage:
    -s address       Rnmap server address
    -p port          Server port (default 3500)
    -f file          Filename to save the scanning result (no stdout)
    -n cmd           Nmap command
    -u user          Username

    --plain          Use plaintext communications
    --help           Prints this message
    --mparseable     Use machine parseable log form
    --scriptkiddie   Use script kiddie log form
    --xml            Use xml log form

    --version        Show version of rnmap cli client

    example:
    # rnmap.py -s 192.168.1.78 -n "-sS -p 1-65535 192.168.1.10" --mparseable"""

from socket import *
import sys
import os
import string
import getopt
import signal
import getpass

sys.path[:0] = ['../lib']

from client import *
from sec import *
if Use_cipher == 1:
    import cPickle

Port = 3500
Mode = " -oN"
File = None
Server = None
Nmapcmd = ""
Crypt = 1

try:
    opts, args = getopt.getopt(sys.argv[1:], 's:p:n:f:u:', ['help', 'version', 'mparseable', 'scriptkiddie', 'xml', 'plain'])
except:
    sys.stderr.write("Try 'rnmap.py --help' for more information.\n")
    sys.exit(1)

for o, a in opts:
    if o == '--version':
        sys.stdout.write("Rnmap cli client v. 0.1.7\n")
        sys.exit()
    if o == '-s': Server = a
    if o == '-p': Port = string.atoi(a)
    if o == '-n':
        Nmapcmd = string.replace(a, ":", "###")
    if o == '-f': File = a
    if o == '-u':
        logid = a
    if o == '--plain': Crypt = 0
    if o == '--mparseable': Mode = " -oM"
    elif o == '--scriptkiddie': Mode = " -oS"
    elif o == '--xml': Mode = " -oX"
    if o == '--help':
        print __doc__
        sys.exit()

if Server == None:
    print __doc__
    sys.exit(1)

def check_cipheruse():
    if Use_cipher == 0:
        sys.stderr.write("Can't find cryptographic modules!\n")
    elif check_randomsrc() == 0:
        sys.stderr.write("Can't find random source!\n")
    else:
        return
    sys.stderr.write("Use --plain command line option to force plaintext communications mode if you know what you are doing.\n")
    sys.exit(1)
    
def connection(packet):
    s = socket(AF_INET, SOCK_STREAM)
    signal.alarm(10)
    try:
        s.connect((Server, Port))
    except error, e:
        if e[0] == 4:
            e = (110, 'Connection timed out')
        sys.stderr.write(e[1] + "\n")
        sys.exit(1)
    signal.alarm(0)
    s.send(packet)
    data = ''
    while 1:
        read = s.recv(4096)
        if not read:
            break
        data = data + read    
    s.close()
    return data

def invalid_key():
    sys.stderr.write("Invalid server key\n")
    sys.exit(1)
    
def ask_username():
    logid = raw_input("Username for " + Server + ": ")
    return logid

def ask_password():
    passwd = getpass.getpass()
    return passwd

def message(logid, passwd):
    if Crypt == 1:
        publickey = fetch_key(Server)
        if publickey == None:
            message = Keyreq + logid
        else:
            message, key = generate_message(logid, passwd, publickey, Nmapcmd + Mode)
    else:
        message = "100:" + logid + ":" + passwd + ":" + Nmapcmd + Mode
        
    result = connection(message)

    if Crypt == 1 and publickey  == None and result[:3] != "405":
        spkey = get_pubkey(passwd, result)
        try:
            publickey = cPickle.loads(spkey[4:])
        except:
            invalid_key()
        else:
            save_key(spkey[4:], Server)
            message, key = generate_message(logid, passwd, publickey, Nmapcmd + Mode)
            result = connection(message)
                
    if Crypt == 1 and result[:3] not in ("401", "405", "407"):
        iv = result[-8:]
        result = decipher(result[:-8], key, iv)
    if result == None or result[:3] not in Statusheaders:
        sys.stdout.write(Perror + "\n")
    else:
        if File == None:
            sys.stdout.write(result[4:])
            if result[:3] != "201":
                sys.stdout.write("\n")
        else:
            if result[:3] != "201":
                sys.stdout.write(result[4:] + "\n")
            else:
                try:
                    fp = open(File, "w")
                    fp.write(result[4:])
                    fp.close()
                except IOError:
                    sys.stderr.write("\nCan't write to disk!\n")
                else:
                    sys.stdout.write("\n")

if Crypt == 1:
    check_cipheruse()
os.umask(0066)
signal.signal(signal.SIGALRM, signal_handler)

try:
    if logid:
        passwd = ask_password()
except NameError:
    logid = ask_username()
    passwd = ask_password()
    
message(logid, passwd)
